use strict;

if ($#ARGV != 1)
{
    print "Usage: $0 <test_file> <gamma>\n";
    exit;
}

my $test_file = $ARGV[0];
my $gamma = $ARGV[1];

my $testname;
if ($test_file =~ /\/([^\/]+)\.txt/)
{
    $testname = $1;
}

# CHANGE this to your path of contrafold
my $contrafold = "~/bio/Software/contrafold_2.0/src/contrafold";
my $tempi = "/tmp/cinput-$testname-$gamma.txt";      # temporary input file to contrafold (assuming this is run once at the same time
my $tempo = "/tmp/coutput-$testname-$gamma.txt";     # temporary output file of contrafold (assuming this is run once at the same time


my $line;
my $known_structure;
my $predicted_structure;
my $num = 0;
my $avgsens = 0;
my $avgppv = 0;
my $avgf = 0;
open (FILE, $test_file) || die ("Cannot open $test_file");
while ($line = <FILE>)
{
    if ($line =~ /^>/)
    {
        open (TMP, ">$tempi") || die("Cannot open file");
        print TMP "$line";
        print $line;        
        $line = <FILE>;     # sequence
        print TMP "$line";
        print $line;        
        close(TMP);
        # now run contrafold
        system ("$contrafold predict --gamma $gamma --parens $tempo $tempi");
        $known_structure = <FILE>;
        print $known_structure;
        chomp($known_structure);
        # get the contrafold structure
        $predicted_structure = `tail -n -1 $tempo`;
        print $predicted_structure;
        chomp ($predicted_structure);
        my ($sens, $ppv, $f) = get_accuracy ($known_structure, $predicted_structure);
        printf ("sens=%.3f, ppv=%.3f, f=%.3f\n", $sens, $ppv, $f);
        $num++;
        $avgsens += $sens;
        $avgppv += $ppv;
        $avgf += $f;
        print "\n";
    }
}
close (FILE);
$avgsens /= $num;
$avgppv /= $num;
$avgf /= $num;

printf ("Total num=$num, avg_sens=%.3f, avg_ppv=%.3f, avg_f=%.3f\n\n", $avgsens, $avgppv, $avgf);

#############################################
sub get_accuracy
{
    my ($known, $predicted) = @_;
    die ("known and predicted structure must have the same length!") if (length($known) != length($predicted));
    my @tknown = get_ptable ($known);
    my @pknown = get_ptable ($predicted);    
    my $i;
    my $sens = 0;
    my $ppv = 0;
    my $f = 0;
    
    my $num_corr_pairings = 0;
    my $num_true_pairings = 0;
    my $num_pred_pairings = 0;
    for ($i=0; $i <= $#tknown; $i++)
    {
        $num_pred_pairings++ if ($pknown[$i] > -1);
        $num_true_pairings++ if ($tknown[$i] > -1);
        $num_corr_pairings++ if ($pknown[$i] > -1 && $pknown[$i]==$tknown[$i]);
    }
    $sens = $num_corr_pairings / $num_true_pairings if ($num_true_pairings != 0);
    $ppv = $num_corr_pairings / $num_pred_pairings  if ($num_pred_pairings != 0);
    $f = 2*$sens*$ppv/($ppv+$sens)   if ($ppv+$sens != 0);
    return ($sens, $ppv, $f);
}

#############################################

sub get_ptable
{
    my ($structure) = @_;
    my @p_table;
    my $length_structure = length ($structure);
    my $i; my $j;
    my @stack;
    for ($i=0; $i<$length_structure; $i++)
      {
        my $pos = substr($structure, $i, 1);
        if ($pos eq "." || $pos eq ":")
          {
            $p_table[$i] = "-1";
          }
        elsif ($pos eq "(")
          {
            push (@stack, $i);
          }
        elsif ($pos eq ")" || $pos eq ">")
          {
            $j = pop (@stack);
            $p_table[$i] = $j;
            $p_table[$j] = $i;
          }
        else {
            die "*** Unknown character: $pos\n";
        }
      }
    return @p_table;
}