#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include "simfold.h"
#include "common.h"
#include "params.h"
#include "externs.h"

#define MAXL 5000


int main (int argc, char *argv[])
{
    char sequence[MAXL];
    char known_structure[MAXL];
    char pred_structure[MAXL];
    char test_filename[1000];
    char paramfile[1000];
    char options_model_file[1000];
    FILE *test_file;
    int i, j;
    char buffer[MAXL];    
    double energy;
    
    double sens=0, avg_sens=0;
    double ppv=0, avg_ppv=0;
    double f=0, avg_f=0;
    int num=0;
    
        
    if (argc != 4)
    {
        printf ("Usage: %s <test_file> <options_model_file> <parameter_file>\n", argv[0]);
        exit(0);
    }
    strcpy (test_filename, argv[1]);    
    strcpy (options_model_file, argv[2]);
    strcpy (paramfile, argv[3]);

    if (strcmp (options_model_file, "none") != 0)
    {
        read_parsi_options_from_file (options_model_file);
    }
    
    char config_file[200] = "../../../Projects/MultiRNAFold/params/multirnafold.conf";
    char thermo_filename[] = "../data/RNA-thermo-db_v1.3.xml";
    init_data (argv[0], config_file, RNA, 37.0);       
    
    num_params = create_string_params();    

    fill_data_structures_with_new_parameters (paramfile);

    // this is only needed for the case when the number of parameters in the model is not 363
    if (strcmp (thermo_filename, "none") != 0)
    {
        // I need this, otherwise I don't know which params are "experimental additions" and which not -- could be taken from a file, of course
        num_params = create_string_params();
        set_starters();
        creating_model = 1;
        fill_similarity_rule_with_optical_melting_reference (thermo_filename);
        creating_model = 0;
        fill_data_structures_with_new_parameters (paramfile);
        num_params = create_string_params();
        /////////////
    }

    
       
    //////////////////////////////////////////////////////
    // open the old set, to get the sequence, and the structure 
    if ((test_file = fopen(test_filename, "r")) == NULL)
    {
        printf ("Cannot open file %s\n", test_filename);
        exit(0);
    }
               
    fgets (buffer, sizeof(buffer), test_file);    // header "> SSTRAND_ID etc"
    while (!feof(test_file))
    {      
        // make sure this is the header
        if (buffer[0] != '>')   
        {
            fgets (buffer, sizeof(buffer), test_file); 
            continue;        
        }
        //printf (newset_file, "%s", buffer);
        fgets (sequence, sizeof(buffer), test_file);    // sequence
        //fprintf (newset_file, "%s", sequence);
        fgets (known_structure, sizeof(buffer), test_file);    // known structure
        //fprintf (newset_file, "%s", known_structure);
        for (i=0; i < strlen (sequence); i++)
        {
            if(sequence[i] == '\n')
            {
                sequence[i] = '\0';
                known_structure[i] = '\0';
                break;
            }
        }                                     

        energy = simfold (sequence, pred_structure);  
        printf ("%s%s\n%s\n%s\n", buffer, sequence, known_structure, pred_structure);
        sens = compute_sensitivity (known_structure, pred_structure);
        ppv = compute_ppv (known_structure, pred_structure);
        num++;
        if (sens+ppv == 0)  f = 0;
        else  f = 2.0*sens*ppv/(sens+ppv);
        avg_sens += sens;
        avg_ppv += ppv;
        avg_f += f;
        
        printf ("energy=%.3lf, sens=%.3lf, ppv=%.3lf, f=%.3lf\n\n", 
                energy, sens, ppv, f);
        
        fgets (buffer, sizeof(buffer), test_file); 
        
    }
        
    fclose (test_file);      
    avg_sens /= num;
    avg_ppv /= num;
    avg_f /= num;

    printf ("Total num=%d, avg_sens=%.3lf, avg_ppv=%.3lf, avg_f=%.3lf\n\n", 
                num, avg_sens, avg_ppv, avg_f);

}
