# given as input a prediction file that has:
# header
# sequence
# known structure
# predicted structure
# prediction info: energy=-101.403, sens=0.676, ppv=0.590, f=0.630
# get the average f-measure per RNA classes

use strict;

if ($#ARGV != 1)
{
    print "Usage: $0 <prediction_file> <show_types:0|1>\n";
    # for the leftmost column, run it with show_types 1, then run it with show_types 0 for the rest of the columns, and then use paste.
    exit;
}

my $prediction_file = $ARGV[0];
my $show_types = $ARGV[1];
my $line;
my $strand_id;
my $type;
my $i;

my @wanted_types = ("Transfer RNA","Ribonuclease P RNA","Signal Recognition Particle RNA","Transfer Messenger RNA","16S Ribosomal RNA","5S Ribosomal RNA","Hammerhead Ribozyme","Group I Intron","23S Ribosomal RNA","Ciliate Telomerase RNA");

my @no      = ();     # one more, for other
my @lenavg  = (); 
my @lensq   = (); 
my @lenstd  = ();    # compute this at the end
my @sens    = ();
my @ppv     = ();    
my @f       = ();    # compute this at the end

# initialize all arrays with 0
# the last one is the entire set
# the one before the last is "Other"
for ($i=0; $i <=$#wanted_types+2; $i++)
{
    push (@no, 0);
    push (@lenavg, 0);
    push (@lensq, 0);
    push (@lenstd, 0);
    push (@sens, 0);
    push (@ppv, 0);
    push (@f, 0);
}

my $allindex = $#wanted_types+2;

open (FILE, $prediction_file) || die ("Cannot open $prediction_file");
while ($line = <FILE>)
{
    if ($line =~ /> SSTRAND_ID=(\S+); TYPE=(.*); EXT_SOURCE/)
    {
        $strand_id = $1;
        $type = $2;
        #print "$strand_id\t\t$type\n";
        for ($i=0; $i <= $#wanted_types; $i++)
        {
            if ($type eq $wanted_types[$i])     { last; }
        }   # now $i is the index 
        $line = <FILE>;     # this is the sequence
        chomp ($line);
        # now get the statistics of the sequences
        $no[$i]++;
        $lenavg[$i] += length($line);
        $lensq[$i] += length($line)*length($line);
        
        # allso fill up the statistics for all
        $no[$allindex]++;
        $lenavg[$allindex] += length($line);
        $lensq[$allindex] += length($line)*length($line);        
        
        $line = <FILE>;     # known structure
        $line = <FILE>;     # predicted structure
        $line = <FILE>;     # the accuracy line
        if ($line =~ /f=(\S+)/)     # I can also get sens and ppv if needed
        {
            $f[$i] += $1;
            $f[$allindex] += $1;
        }      
    }
}
close (FILE);
for ($i=0; $i <=$#wanted_types+2; $i++)
{
    $lenavg[$i] /= $no[$i];
    $lenstd[$i] = sqrt(1/$no[$i]*$lensq[$i] - $lenavg[$i]*$lenavg[$i]);
#     $sens[$i] /= $nosens[$i];
#     $ppv[$i] /= $noppv[$i];
    $f[$i] /= $no[$i];
    if ($i <= $#wanted_types)       {   $type = $wanted_types[$i];  }
    elsif ($i == $#wanted_types+1)  {   $type = "Other";    }
    else                            {   $type = "S-STRAND2";    }
    if ($show_types)
    {
        printf ("%30s &\t%d &\t%.1f\t\$\\pm\$\t%.1f &\t%.3f\n", $type, $no[$i],$lenavg[$i],$lenstd[$i],$f[$i]);        
    }
    else
    {
        printf (" &\t%.3f\n", $f[$i]);
    }
}

