use strict;
use Cwd;


if ($#ARGV != 2)
{
    print "Usage: $0 <input_test_dir> <output_dp_dir> <param_set>\n";
    #print "<input_dir> should include pattern, e.g. dir/* or dir/S*\n";
    exit;
}

my $simfold = "get_parameters_accuracy";
my $sm = 0;

my $working_dir = getcwd();


my $input_dir = $ARGV[0];
my $output_dir = $ARGV[1];
my $param = $ARGV[2];
# my $output_file = "$output_dir/results.txt";
my $sge_script = "test_simfold_sge.sh";

# create the output file if it doesn't exist
if (! -e $output_dir)
{
    system ("mkdir $output_dir");
}

my $truefile;
my $command;
my $sens;
my $ppv;
my $acc;


my $avg_sens = 0;
my $avg_ppv = 0;
my $avg_acc = 0;
my $num_sens = 0;
my $num_ppv = 0;
my $num_acc = 0;

my $dp_structure;
my @ptable = ();
#my $predfile;
my $predfile_dp;
my $file_short;
my $seqfile;

my $index = 0;


# open (OUT, ">$output_file") || die ("Cannot open $output_file");

foreach $truefile (glob ("$input_dir/*.bpseq")) {
    my @items = split(/\//, $truefile);
    $file_short = $items[$#items];
    $file_short =~ s/\.bpseq//;

    $index++;

    # first fold and get dp structure
    #$predfile = "$output_dir/${file_short}.bpseq";
    $predfile_dp = "$output_dir/${file_short}.dp";
    if (-e $predfile_dp)
    {  next;  }

    $seqfile = "$output_dir/SSEEQ${index}_${file_short}.seq";
    bpseq2seq ($truefile, $seqfile);

    $command = "qsub -cwd -o /dev/null -e /dev/null $sge_script $seqfile $param $predfile_dp";
    print "$command\n";
    system ($command);
    
    #system ("rm -rf $seqfile");
    # convert dp into ptable



    # doesn't make sense to analyse here, because the files aren't ready, should be done in a separate file
#     open (DP, $predfile_dp) || die ("Cannot open $predfile_dp");
#     $dp_structure = <DP>;
#     chomp ($dp_structure);
#     close (DP);
#     @ptable = ();    
#     @ptable = get_ptable ($dp_structure);
# 
#     ($sens, $ppv, $acc) = measure_sensitivity_and_ppv_ptable ($truefile, @ptable);
#     if ($sens != -1) {  $avg_sens += $sens; $num_sens++;}
#     if ($ppv != -1) {  $avg_ppv += $ppv; $num_ppv++}
#     $avg_acc += $acc;
#     $num_acc++;
#     printf (OUT "%s\tacc = %.2f\tsens = %.2f\tppv = %.2f\n", $file_short, $acc, $sens, $ppv);
    #printf ("\tavg_acc = %.2f\tavg_sens = %.2f\tavg_ppv = %.2f\n", $avg_acc/$num, $avg_sens/$num, $avg_ppv/$num);
    #last;
}

# now run the array job

# no idea why this doesn't work
#$command = "qsub -cwd -o /dev/null -e /dev/null -t 1-$index $sge_script $param $output_dir";
#print "$command\n";
#system ($command);


# $avg_sens /= $num_sens;
# $avg_ppv /= $num_ppv;
# $avg_acc /= $num_acc;
# printf (OUT "==============================\n");
# printf (OUT "Avg sens = %.2f, Avg ppv = %.2f, Avg acc = %.2f, input_dir = %s, params = %s\n", $avg_sens, $avg_ppv, $avg_acc, $input_dir, $param);
#print "Gamma = $gamma, Avg sens = $avg_sens, Avg ppv = $avg_ppv, dir = $dir, params = $param\n";


# clean up
#system ("perl remove_files.pl \"~/test_simfold_sge.sh.*\"");


#############################################
sub measure_sensitivity_and_ppv_ptable {
    # also measure accuracy
    my ($true_bpseq, @ptable) = @_;    

    my $sens;    
    my $ppv;
    my $acc;
    my $pred_line;
    my $true_line;
    my $num_corr_pairings = 0;
    my $num_true_pairings = 0;
    my $num_pred_pairings = 0;
    my $distance = 0;
    my $len = 0;
    my $pred;
    my $true;
    
    
    open (TRUE, $true_bpseq) || die ("Cannot open $true_bpseq");
    while ($true_line = <TRUE>) {
        $pred = $ptable[$len]+1;
        $len++;
        if ($true_line =~ /\d+\s\S+\s(\d+)/) {
            $true = $1;
        }        
        if ($pred > 0) {
            $num_pred_pairings++;
        }
        if ($true > 0) {
            $num_true_pairings++;
        }        
        if ($pred == $true && $pred > 0) {
            $num_corr_pairings++;
        }
        if ($pred != $true) {
            $distance++;
        }
        #print "$pred $true\n";
    }
    close (TRUE);
    #print "$num_corr_pairings, $num_true_pairings, $num_pred_pairings\n";
    if ($num_true_pairings > 0) {
        $sens = $num_corr_pairings / $num_true_pairings;
    }
    else { $sens = -1; }    
    if ($num_pred_pairings > 0) {
        $ppv = $num_corr_pairings / $num_pred_pairings;
    }
    else { $ppv = -1; }
    $acc = 1-$distance/$len;
    return ($sens, $ppv, $acc);
}


#############################################

sub bpseq2seq {
    my ($bpseq_file, $seq_file) = @_;
    my $line;
    my $sequence = "";
    open (FILE, $bpseq_file) || die ("Cannot open $bpseq_file");
    while ($line = <FILE>) {
        if ($line =~ /^\n/)  { next; }
        if ($line =~ /(\d+)\s(\S+)\s(\d+)/) {
            $sequence .= $2;
        }
        else {
            print "Wrong format: $line";
        }
    }
    close (FILE);
    open (SEQ, ">$seq_file") || die ("Cannot open $seq_file");
    print SEQ "$sequence\n";
    close (SEQ);
}

#############################################

sub get_ptable
  {
    my ($structure) = @_;
    my @p_table;
    my $length_structure = length ($structure);
    my $i; my $j;
    my @stack1;
    my @stack2;
    my @stack3;
    for ($i=0; $i<$length_structure; $i++)
      {
        my $pos = substr($structure, $i, 1);
        if ($pos eq "." || $pos eq ":")
          {
            $p_table[$i] = "-1";
          }
        elsif ($pos eq "(" || $pos eq "<")
          {
            push (@stack1, $i);
          }
        elsif ($pos eq "[" || $pos eq "A")
          {
            push (@stack2, $i);
          }
        elsif ($pos eq "{")
          {
            push (@stack3, $i);
          }
        elsif ($pos eq ")" || $pos eq ">")
          {
            $j = pop (@stack1);
            $p_table[$i] = $j;
            $p_table[$j] = $i;
          }
        elsif ($pos eq "]" || $pos eq "a")
          {
            $j = pop (@stack2);
            $p_table[$i] = $j;
            $p_table[$j] = $i;
          }
        elsif ($pos eq "}")
          {
            $j = pop (@stack3);
            $p_table[$i] = $j;
            $p_table[$j] = $i;
          }
        else {
            print "*** Unknown character: $pos\n";
        }  
      }
    return @p_table;
  }

